@@ -681,9 +681,12 @@ class CallSet:
             df = v.breaks_df
             if df is not None and ol_tree is not None:
                 for index, chrom, start, chrom2, end in zip(df.index, df.chrom, df.start, df.chrom2, df.end):
-                    if ol_tree and chrom in ol_tree:
-                        if not any(ol_tree[chrom].ncls.find_overlap(start, start + 1)) or not any(ol_tree[chrom2].ncls.find_overlap(end, end + 1)):
+                    if ol_tree:
+                        if chrom not in ol_tree:
                             bad_i.add(index)
+                        else:
+                            if not any(ol_tree[chrom].ncls.find_overlap(start, start + 1)) or not any(ol_tree[chrom2].ncls.find_overlap(end, end + 1)):
+                                bad_i.add(index)
             v.breaks_df = v.breaks_df.drop(bad_i)
 
         print("Filtered by include_bed, caller={}, dataset={} rows before {}, after {}".format(v.caller, v.dataset,
@@ -844,6 +847,8 @@ class CallSet:
                 vcf_index += 1
             except StopIteration:
                 break
+            except IndexError:
+                raise IndexError("last record was", str(r))
 
             ol_start = False
             ol_end = False
@@ -855,6 +860,9 @@ class CallSet:
             if allowed_chroms is not None and chrom not in allowed_chroms:
                 continue
 
+            if "#" in chrom:
+                continue
+
             start = int(r.POS)
 
             if ol_tree and chrom in ol_tree:
@@ -864,10 +872,22 @@ class CallSet:
                     not_in_include += 1
                     continue
 
-            try:
+            if "SVTYPE" in r.INFO:
                 svtype = r.INFO["SVTYPE"] if not isinstance(r.INFO["SVTYPE"], list) else r.INFO["SVTYPE"][0]
-            except KeyError:
-                continue
+            else:
+                if isinstance(r.REF, list):
+                    lr = len(r.REF[0])
+                else:
+                    lr = len(r.REF)
+                if isinstance(r.ALT, list):
+                    la = len(r.ALT[0])
+                else:
+                    la = len(r.ALT)
+                if lr > 1 or la > 1:
+                    if lr > la:
+                        r.INFO["SVTYPE"] = "DEL"
+                    else:
+                        r.INFO["SVTYPE"] = "INS"
 
             try:
                 if allowed_svtypes is not None and svtype not in allowed_svtypes:
@@ -885,12 +905,22 @@ class CallSet:
             if svtype == "BND":
                 if "_2" in r.ID:  # Skip second part of BND record
                     continue
-                chrom2 = r.ALT[0].chr
-                end = r.ALT[0].pos
+                if r.ALT[0] is None:
+                    continue
+                try:
+                    chrom2 = r.ALT[0].chr
+                    end = r.ALT[0].pos
+                except AttributeError:
+                    print("Error parsing", r, file=stderr)
+                    continue
                 if end is None:
                     end = start + 1
                 else:
                     end = int(r.ALT[0].pos)
+
+
+
+
             else:
                 chrom2 = chrom
                 if "CHR2" in r.INFO:
@@ -905,22 +935,25 @@ class CallSet:
                         end = end[0]
                     end = int(end)
                     done = True
-                elif svtype == "DEL" and "SVLEN" in r.INFO:
+                elif svtype in ("DEL", "DUP", "INV") and "SVLEN" in r.INFO:
                     svlen = r.INFO["SVLEN"]
                     if isinstance(svlen, list):
                         svlen = svlen[0]
                     end = start + svlen
                     done = True
                 else:  # Try and use ALT / REF lengths
-                    if r.INFO["SVTYPE"] == "DEL":
+
+                    if r.INFO["SVTYPE"] == "DEL" or (isinstance(r.INFO["SVTYPE"], list) and r.INFO["SVTYPE"][0] == "DEL"):
                         svlen = len(r.REF) if r.ALT is not isinstance(r.REF, list) else r.REF[0]
+                        end = r.POS + svlen
                         done = True
-                    elif r.INFO["SVTYPE"] == "INS":
+                    elif r.INFO["SVTYPE"] == "INS" or (isinstance(r.INFO["SVTYPE"], list) and r.INFO["SVTYPE"][0] == "INS"):
                         svlen = len(r.ALT) if r.ALT is not isinstance(r.ALT, list) else r.ALT[0]
+                        end = r.POS + svlen
                         done = True
 
                 if not done:
-                    print("Warning: could not parse record", r, file=stderr)
+                    print("Warning: could not parse record", r, r.INFO, file=stderr)
                     continue
 
             if no_translocations and chrom != chrom2:
@@ -983,7 +1016,7 @@ class CallSet:
                 unique_ids.add(r_id)
 
             d = {"end": end, "start": start, "chrom": chrom, "chrom2": chrom2, "w": w, "id": r_id, "svtype": svtype,
-                 "size_filter_pass": size_filter, "svlen": abs(svlen) if svlen else svlen}
+                 "size_filter_pass": size_filter, "svlen": abs(svlen) if svlen else svlen, "filter": r.FILTER}
 
             if stratify is not None:
                 d["strata"] = get_strata(r, stratify)
@@ -997,8 +1030,21 @@ class CallSet:
             if load_genotype:
                 samps = r.__getattribute__("samples")
                 if len(samps) > 1:
-                    raise ValueError("Cannot parse genotype for multi-sample vcf")
-                d["GT"] = str(samps[0]["GT"])
+                    raise ValueError("Cannot parse genotype for multi-sample vcf, set load_genotype=False")
+                done = False
+                try:
+                    d["GT"] = str(samps[0]["GT"])
+                    done = True
+                except IndexError:
+                    pass
+
+                if not done:
+                    try:
+                        d["GT"] = r.INFO["GT"]
+                    except KeyError:
+                        pass
+                    d["GT"] = "NA"
+
             res.append(d)
 
         if len(res) == 0:
@@ -1018,9 +1064,10 @@ class CallSet:
                         df[ec] = v.norm(df[ec], self.kwargs)
 
         # Order df
-        base_cols = ["chrom", "start", "chrom2", "end", "svtype", "w", "strata", "id", "size_filter_pass", "svlen"]
+        base_cols = ["chrom", "start", "chrom2", "end", "svtype", "w", "strata", "id", "size_filter_pass", "svlen", "filter"]
         if load_genotype:
             base_cols.append("GT")
+            df["GT"] = df["GT"].str.replace("|", "/")
         df = df[[i for i in base_cols if i in df.columns] + new_cols]
 
         self.breaks_df = df
@@ -1438,7 +1485,7 @@ def best_index(out_edges, key="q"):
 
 def quantify(ref_data, data, force_intersection=False, reciprocal_overlap=0., show_table=True, stratify=False,
              good_indexes_only=False, ref_size_bins=(30, 50, 500, 5000, 260000000), allow_duplicate_tp=True,
-             pct_size=0.05, ignore_svtype=True, min_ref_size=20, max_ref_size=None):
+             pct_size=0.05, ignore_svtype=True, min_ref_size=20, max_ref_size=None, dups_and_ins_equivalent=False):
 
     # Build a Maximum Bipartite Matching graph:
     # https://www.geeksforgeeks.org/maximum-bipartite-matching/
@@ -1470,7 +1517,6 @@ def quantify(ref_data, data, force_intersection=False, reciprocal_overlap=0., sh
         chrom, start, chrom2, end = sv_key(chrom, start, chrom2, end)
 
         ol_start = intersecter(tree, chrom, start, start + 1)
-
         if not ol_start:
             continue
 
@@ -1506,9 +1552,10 @@ def quantify(ref_data, data, force_intersection=False, reciprocal_overlap=0., sh
                 continue
 
             if not ignore_svtype and ref_row["svtype"] != svtype:
-                continue
-
-            # if svtype != "INS":
+                if dups_and_ins_equivalent and svtype in ("DUP", "INS") and ref_row["svtype"] in ("DUP", "INS"):
+                    pass
+                else:
+                    continue
 
             # If intra-chromosomal, check reciprocal overlap
             if chrom == chrom2:
@@ -1756,7 +1803,7 @@ def quantify(ref_data, data, force_intersection=False, reciprocal_overlap=0., sh
         gt_correct = []
         actual_gt = []
         for i, q_gt in zip(data.breaks_df.ref_index, data.breaks_df.GT):
-            if i == i:
+            if i == i and i is not None:
                 r_gt = ref_gts.loc[int(i)]
                 if q_gt == r_gt:
                     gt_correct.append(True)
